import pl.edu.s3996.zad2.InvalidMessageException;
import protocol.*;

import java.io.IOException;
import java.lang.reflect.Type;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;


public class Client {
    private static SocketChannel channel;
    private static Charset charset = Charset.forName("ISO-8859-2");
    private static ByteBuffer inBuf = ByteBuffer.allocateDirect(1024);

    public static void main(String[] args) {
        try {
            channel = SocketChannel.open();
            channel.configureBlocking(false);


            connect();
            EchoRequestMessage echoRequestMessage = new EchoRequestMessage();
            echoRequestMessage.setMessageToReply("Hello SERVER!");
            System.out.println("Client: " + echoRequestMessage);
            sendMessage(echoRequestMessage);

            connect();
            AddRequestMessage addRequestMessage = new AddRequestMessage();
            addRequestMessage.setFirstNumber(10);
            addRequestMessage.setSecondNumber(5);

            System.out.println("Client: " + addRequestMessage);
            sendMessage(addRequestMessage);

        } catch (UnknownHostException exc) {
            System.exit(1);
        } catch (IOException exc) {
            exc.printStackTrace();
            System.exit(2);
        }

    }

    private static void connect() throws IOException {
        if (!channel.isOpen()) channel = SocketChannel.open();
        channel.connect(new InetSocketAddress(InetAddress.getLocalHost(), 7564));
        System.out.print("Łącze się ...");
        while (!channel.finishConnect()) {
            try {
                Thread.sleep(200);
            } catch (Exception exc) {
                return;
            }
        }
        System.out.println("Połączony.");
    }


    private static void sendMessage(BaseMessage message) {
        StringBuffer result = new StringBuffer();
        try {
            CharBuffer cbuf = CharBuffer.wrap(message.saveToString() + "\r\n");
            ByteBuffer  outBuf = charset.encode(cbuf);
            channel.write(outBuf);

            while (true) {
                inBuf.clear();
                int readBytes = channel.read(inBuf);
                if (readBytes == 0) {
                    Thread.sleep(200);
                    continue;
                } else if (readBytes == -1) {
                    System.out.println("Kanał zamknięty");
                    channel.close();
                    break;
                } else {
                    inBuf.flip();
                    cbuf = charset.decode(inBuf);
                    result.append(cbuf);
                    System.out.println("Dostałem " + result);

                    Type messageType = protocol.BaseMessage.getMessageType(result.toString());

                    protocol.BaseMessage recievedMessage;

                    if (messageType.equals(EchoResponseMessage.class)) {
                        recievedMessage = new EchoResponseMessage();
                        recievedMessage.parseFromString(result.toString());
                        System.out.println("Echo response: " + ((EchoResponseMessage)recievedMessage).getMessage());
                    } else if (messageType.equals(AddResponseMessage.class)) {
                        recievedMessage = new AddResponseMessage();
                        recievedMessage.parseFromString(result.toString());
                        System.out.println("Add response: " + ((AddResponseMessage) recievedMessage).getSum());
                    }

                    if (message == null) {
                        throw new InvalidMessageException();
                    }
                }
            }
        } catch (Exception exc) {
            exc.printStackTrace();
            return;
        }
    }
}
