import pl.edu.s3996.zad2.InvalidMessageException;
import protocol.AddRequestMessage;
import protocol.BaseMessage;
import protocol.EchoRequestMessage;

import java.lang.reflect.Type;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.util.Iterator;
import java.util.Set;

public class Server {
    private ServerSocketChannel ssc = null;
    private Selector selector = null;
    private static Charset charset = Charset.forName("ISO-8859-2");
    private static final int BUFFER_SIZE = 1024;
    private ByteBuffer byteBuffer = ByteBuffer.allocate(BUFFER_SIZE);
    private static final int PORT_NO = 7564;

    public Server() {
        try {
            ssc = ServerSocketChannel.open();
            ssc.configureBlocking(false);
            ssc.socket().bind(new InetSocketAddress(InetAddress.getLocalHost(), PORT_NO));
            selector = Selector.open();
            ssc.register(selector, SelectionKey.OP_ACCEPT);

        } catch (Exception exc) {
            exc.printStackTrace();
            System.exit(1);
        }
        System.out.println("Server started on port " + PORT_NO);
        serviceConnections();
    }

    private void serviceConnections() {
        boolean serverIsRunning = true;

        while (serverIsRunning) {
            try {
                selector.select();
                Set keys = selector.selectedKeys();
                Iterator iter = keys.iterator();

                while (iter.hasNext()) {
                    SelectionKey key = (SelectionKey) iter.next();
                    iter.remove();
                    if (key.isAcceptable()) {
                        SocketChannel cc = ssc.accept();
                        cc.configureBlocking(false);
                        cc.register(selector, SelectionKey.OP_READ);
                        continue;
                    }

                    if (key.isReadable()) {
                        SocketChannel cc = (SocketChannel) key.channel();

                        processRequest(cc);

                        continue;
                    }
                }
            } catch (Exception exc) {
                exc.printStackTrace();
                continue;
            }
        }
    }


    private void processRequest(SocketChannel sc) {
        if (!sc.isOpen()) return;
        StringBuffer reqString = new StringBuffer();
        byteBuffer.clear();

        try {
            readLoop:
            while (true) {
                int n = sc.read(byteBuffer);
                if (n > 0) {
                    byteBuffer.flip();
                    CharBuffer cbuf = charset.decode(byteBuffer);
                    while (cbuf.hasRemaining()) {
                        char c = cbuf.get();
                        if (c == '\r' || c == '\n') {
                            break readLoop;
                        }
                        reqString.append(c);
                    }
                }
            }

            Type messageType = BaseMessage.getMessageType(reqString.toString());

            BaseMessage message = null;

            if (messageType.equals(EchoRequestMessage.class)) {
                message = new EchoRequestMessage();
                message.parseFromString(reqString.toString());
            } else if (messageType.equals(AddRequestMessage.class)) {
                message = new AddRequestMessage();
                message.parseFromString(reqString.toString());
            }

            if (message == null) {
                throw new InvalidMessageException();
            }

            BaseMessage response = message.getResponse();

            ByteBuffer buf = charset.encode(CharBuffer.wrap(response.saveToString()));
            sc.write(buf);
            sc.close();

        } catch (Exception exc) {
            exc.printStackTrace();
            try {
                sc.close();
                sc.socket().close();
            } catch (Exception e) {
            }
        }
    }

    public static void main(String[] args) {
        new Server();
    }
}
